// Baixa Tot WebExtension back end

function encode(data) {
	let bytes = new Uint8Array(data);

	let b = "";
	for (let i = 0; i < bytes.byteLength; i++)
		b += String.fromCharCode(bytes[i]);
	return btoa(b);
}

function listener(details) {
	let filter = browser.webRequest.filterResponseData(details.requestId);
	let port;

	filter.onstart = event => {
		log(`Downloading '${details.url}'`);

		port = browser.runtime.connectNative("baixa_tot");

		port.onDisconnect.addListener(p => {
			log("Native back end disconnected");
		});

		port.onMessage.addListener(response => {
			if (response.result) {
				if (response.result == "error") {
					log("Back end error");
					port.disconnect();
				}
				else if (response.result == "ok")
					port.disconnect();
			}
		});

		port.postMessage({
			command: "initialize",
			debug: WEOptions_debug
		});

		port.postMessage({
			command: "open_file",
			url: details.url
		});
	};

	filter.ondata = event => {
		let encoded = encode(event.data);

		port.postMessage({
			command: "write_data",
			data: encoded
		});

		filter.write(event.data);
	};

	filter.onstop = event => {
		log(`Downloaded '${details.url}'`);

		port.postMessage({
			command: "close_file"
		});

		filter.close();
	};
}

browser.webRequest.onBeforeRequest.addListener(
	listener,
	{urls: ["<all_urls>"]},
	["blocking"]
);
